@@ -92,8 +92,6 @@ struct _CamelEwsFolderPrivate {
 	GHashTable *uid_eflags;
 };
 
-extern gint camel_application_is_exiting;
-
 static gboolean ews_delete_messages (CamelFolder *folder, const GSList *deleted_items, gboolean expunge, GCancellable *cancellable, GError **error);
 static gboolean ews_refresh_info_sync (CamelFolder *folder, GCancellable *cancellable, GError **error);
 
@@ -1409,6 +1407,7 @@ static void
 sync_updated_items (CamelEwsFolder *ews_folder,
                     EEwsConnection *cnc,
                     GSList *updated_items,
+		    CamelFolderChangeInfo *change_info,
                     GCancellable *cancellable,
                     GError **error)
 {
@@ -1422,9 +1421,20 @@ sync_updated_items (CamelEwsFolder *ews_folder,
 
 	for (l = updated_items; l != NULL; l = g_slist_next (l)) {
 		EEwsItem *item = (EEwsItem *) l->data;
-		const EwsId *id = e_ews_item_get_id (item);
+		const EwsId *id;
 		CamelMessageInfo *mi;
 
+		if (!item)
+			continue;
+
+		id = e_ews_item_get_id (item);
+		if (!id) {
+			g_warning ("%s: Missing ItemId for item type %d (subject:%s)", G_STRFUNC, e_ews_item_get_item_type (item),
+				e_ews_item_get_subject (item) ? e_ews_item_get_subject (item) : "???");
+			g_object_unref (item);
+			continue;
+		}
+
 		/* Compare the item_type from summary as the updated items seems to
 		 * arrive as generic types while its not the case */
 		mi = camel_folder_summary_get (folder->summary, id->id);
@@ -1467,7 +1477,7 @@ sync_updated_items (CamelEwsFolder *ews_folder,
 		e_ews_additional_props_free (add_props);
 	}
 
-	camel_ews_utils_sync_updated_items (ews_folder, items);
+	camel_ews_utils_sync_updated_items (ews_folder, items, change_info);
 	items = NULL;
 	if (local_error) {
 		camel_ews_store_maybe_disconnect (ews_store, local_error);
@@ -1490,7 +1500,7 @@ sync_updated_items (CamelEwsFolder *ews_folder,
 
 		e_ews_additional_props_free (add_props);
 	}
-	camel_ews_utils_sync_updated_items (ews_folder, items);
+	camel_ews_utils_sync_updated_items (ews_folder, items, change_info);
 
 	if (local_error) {
 		camel_ews_store_maybe_disconnect (ews_store, local_error);
@@ -1513,6 +1523,7 @@ static void
 sync_created_items (CamelEwsFolder *ews_folder,
                     EEwsConnection *cnc,
                     GSList *created_items,
+		    CamelFolderChangeInfo *change_info,
                     GCancellable *cancellable,
                     GError **error)
 {
@@ -1533,6 +1544,14 @@ sync_created_items (CamelEwsFolder *ews_folder,
 
 		id = e_ews_item_get_id (item);
 		item_type = e_ews_item_get_item_type (item);
+
+		if (!id) {
+			g_warning ("%s: Missing ItemId for item type %d (subject:%s)", G_STRFUNC, item_type,
+				e_ews_item_get_subject (item) ? e_ews_item_get_subject (item) : "???");
+			g_object_unref (item);
+			continue;
+		}
+
 		/* created_msg_ids are items other than generic item. We fetch them
 		 * separately since the property sets vary */
 		/* FIXME: Do we need to handle any other item types
@@ -1575,7 +1594,7 @@ sync_created_items (CamelEwsFolder *ews_folder,
 		goto exit;
 	}
 
-	camel_ews_utils_sync_created_items (ews_folder, cnc, items, cancellable);
+	camel_ews_utils_sync_created_items (ews_folder, cnc, items, change_info, cancellable);
 	items = NULL;
 
 
@@ -1601,7 +1620,7 @@ sync_created_items (CamelEwsFolder *ews_folder,
 		goto exit;
 	}
 
-	camel_ews_utils_sync_created_items (ews_folder, cnc, items, cancellable);
+	camel_ews_utils_sync_created_items (ews_folder, cnc, items, change_info, cancellable);
 	items = NULL;
 
 	if (generic_item_ids) {
@@ -1620,7 +1639,7 @@ sync_created_items (CamelEwsFolder *ews_folder,
 		e_ews_additional_props_free (add_props);
 	}
 
-	camel_ews_utils_sync_created_items (ews_folder, cnc, items, cancellable);
+	camel_ews_utils_sync_created_items (ews_folder, cnc, items, change_info, cancellable);
 
 	if (local_error) {
 		camel_ews_store_maybe_disconnect (ews_store, local_error);
@@ -1688,6 +1707,7 @@ ews_refresh_info_sync (CamelFolder *folder,
                        GCancellable *cancellable,
                        GError **error)
 {
+	CamelFolderChangeInfo *change_info;
 	CamelEwsFolder *ews_folder;
 	CamelEwsFolderPrivate *priv;
 	EEwsConnection *cnc;
@@ -1696,6 +1716,7 @@ ews_refresh_info_sync (CamelFolder *folder,
 	gchar *id;
 	gchar *sync_state;
 	gboolean includes_last_item = FALSE;
+	gint64 last_folder_update_time;
 	GError *local_error = NULL;
 
 	full_name = camel_folder_get_full_name (folder);
@@ -1722,6 +1743,8 @@ ews_refresh_info_sync (CamelFolder *folder,
 
 	camel_folder_summary_prepare_fetch_all (folder->summary, NULL);
 
+	change_info = camel_folder_change_info_new ();
+	last_folder_update_time = g_get_monotonic_time ();
 	id = camel_ews_store_summary_get_folder_id_from_name (
 		ews_store->summary, full_name);
 
@@ -1759,10 +1782,10 @@ ews_refresh_info_sync (CamelFolder *folder,
 		}
 
 		if (items_deleted)
-			camel_ews_utils_sync_deleted_items (ews_folder, items_deleted);
+			camel_ews_utils_sync_deleted_items (ews_folder, items_deleted, change_info);
 
 		if (items_created)
-			sync_created_items (ews_folder, cnc, items_created, cancellable, &local_error);
+			sync_created_items (ews_folder, cnc, items_created, change_info, cancellable, &local_error);
 
 		if (local_error) {
 			if (items_updated) {
@@ -1774,7 +1797,7 @@ ews_refresh_info_sync (CamelFolder *folder,
 		}
 
 		if (items_updated)
-			sync_updated_items (ews_folder, cnc, items_updated, cancellable, &local_error);
+			sync_updated_items (ews_folder, cnc, items_updated, change_info, cancellable, &local_error);
 
 		if (local_error)
 			break;
@@ -1790,9 +1813,27 @@ ews_refresh_info_sync (CamelFolder *folder,
 		((CamelEwsSummary *) folder->summary)->sync_state = sync_state;
 
 		camel_folder_summary_touch (folder->summary);
+
+		if (camel_folder_change_info_changed (change_info)) {
+			camel_folder_summary_save_to_db (folder->summary, NULL);
+			/* Notify any listeners only once per 10 seconds, as such notify can cause UI update */
+			if (g_get_monotonic_time () - last_folder_update_time >= 10 * G_USEC_PER_SEC) {
+				last_folder_update_time = g_get_monotonic_time ();
+				camel_folder_changed (folder, change_info);
+				camel_folder_change_info_clear (change_info);
+			}
+		}
 	} while (!local_error && !includes_last_item && !g_cancellable_is_cancelled (cancellable));
 
-	camel_folder_summary_save_to_db (folder->summary, NULL);
+	if (camel_folder_change_info_changed (change_info)) {
+		camel_folder_summary_touch (folder->summary);
+		camel_folder_summary_save_to_db (folder->summary, NULL);
+		camel_folder_changed (folder, change_info);
+	} else {
+		camel_folder_summary_save_to_db (folder->summary, NULL);
+	}
+
+	camel_folder_change_info_free (change_info);
 
 	if (local_error)
 		g_propagate_error (error, local_error);
@@ -1865,7 +1906,18 @@ ews_append_message_sync (CamelFolder *folder,
 	e_ews_folder_id_free (fid);
 	g_free (folder_id);
 
-	camel_ews_summary_add_message (folder->summary, itemid, changekey, info, message);
+	if (camel_ews_summary_add_message (folder->summary, itemid, changekey, info, message)) {
+		CamelFolderChangeInfo *changes;
+
+		changes = camel_folder_change_info_new ();
+
+		camel_folder_change_info_add_uid (changes, itemid);
+
+		if (camel_folder_change_info_changed (changes))
+			camel_folder_changed (folder, changes);
+
+		camel_folder_change_info_free (changes);
+	}
 
 	if (appended_uid)
 		*appended_uid = itemid;
@@ -1989,8 +2041,8 @@ ews_transfer_messages_to_sync (CamelFolder *source,
 
 			clone = camel_message_info_clone (info);
 
-			camel_ews_summary_add_message (destination->summary, id->id, id->change_key, clone, message);
-			camel_folder_change_info_add_uid (changes, id->id);
+			if (camel_ews_summary_add_message (destination->summary, id->id, id->change_key, clone, message))
+				camel_folder_change_info_add_uid (changes, id->id);
 
 			camel_message_info_unref (clone);
 			camel_message_info_unref (info);
@@ -2372,6 +2424,9 @@ ews_folder_constructed (GObject *object)
 	gchar *host;
 	gchar *user;
 
+	/* Chain up to parent's method. */
+	G_OBJECT_CLASS (camel_ews_folder_parent_class)->constructed (object);
+
 	folder = CAMEL_FOLDER (object);
 	full_name = camel_folder_get_full_name (folder);
 	parent_store = camel_folder_get_parent_store (folder);
